#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>

#include <linux/init.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/pci.h>

#include <asm/uaccess.h>
#include <asm/io.h>

#include <asm/system.h>   /* cli(), *_flags */
#include <asm/segment.h>  /* memcpy and such */
#include <asm/pgtable.h>

#include "bcm.h"

#define DEVNAME "BCM"

static struct bcm_dev *bdev;

struct bcm_transfer trans;

int num=20;
MODULE_PARM(num, "i");

int start=250;
MODULE_PARM(start, "i");

int prot=1;
MODULE_PARM(prot, "i");

void tx_notify(struct bcm_dev *bdev, struct bcm_transfer *tr) {

	printk(KERN_INFO "tx_notified: len=%i\n",tr->len);
	kfree(tr->buffer);
	kfree(tr);

}

void rx_action(struct bcm_dev *bdev, struct bcm_transfer *tr){
	printk(KERN_INFO "rx_acted: len=%i  bytes: %02x %02x %02x %02x\n",
			tr->len,tr->buffer[0],tr->buffer[1],tr->buffer[2],tr->buffer[3]);

	if(tr->len==250) {
		struct bcm_transfer trans;
		int i;
			
		for(i=0;i<num;i++) {
			int j;
			int timeout;
			trans.flags=FLAGS_NOTIFY;
			trans.proto=prot;
			trans.len=1250+i;
			trans.buffer=kmalloc(5000, GFP_KERNEL);
			trans.buffer[0]=i&255;
			trans.buffer[1]=i>>8;
			trans.buffer[2]=66;
			trans.buffer[3]=67;
			trans.dev=bdev;
			j=bcm_transfer_data(&trans);
			timeout=100;
			while(j==-1  && timeout--) {
				printk(KERN_DEBUG "pctest: i=%i bcm_transfer=%i\n", i, j);
				current->state = TASK_INTERRUPTIBLE;
				schedule_timeout (1);
				current->state = TASK_RUNNING;
				j=bcm_transfer_data(&trans);
			}
		}
	}
	kfree(tr->buffer);
	kfree(tr);
}

struct bcm_payload payload={
	1,
	tx_notify,
	rx_action
};

int binit(struct dev_info *dev) {
	int i;
	
	bdev=dev->bcm;
	printk(KERN_INFO "init called, %s\n", dev->identity);
	payload.proto=prot;
	bcm_register_payload(bdev, &payload);
	
	for(i=0;i<num;i++) {
		int j, timeout;
		trans.flags=FLAGS_NOTIFY;
		trans.proto=prot;
		trans.len=start+i;
		trans.buffer=kmalloc(5000, GFP_KERNEL);
		trans.buffer[0]=i&255;
		trans.buffer[1]=i>>8;
		trans.buffer[2]=66;
		trans.buffer[3]=67;
		trans.dev=bdev;
		j=bcm_transfer_data(&trans);
		timeout=100;
		while(j==-1 && timeout--) {
			printk(KERN_DEBUG "pctest: i=%i bcm_transfer=%i\n", i, j);
			current->state = TASK_INTERRUPTIBLE;
			schedule_timeout (1);
			current->state = TASK_RUNNING;
			j=bcm_transfer_data(&trans);
		}
		
	}
	return 0;
}

static struct bcm_driver driver= {
	binit,
	NULL
};

static int __init s(void) {
	bcm_register_driver(&driver);	
	return 0;
}

static void __exit e(void) {
	bcm_register_payload(bdev, &payload);
	return;
}

module_init(s);
module_exit(e);
MODULE_LICENSE("GPL");
